@@ -1,6 +1,5 @@
 import os
 import sys
-import time
 import shutil
 import tempfile
 import subprocess as sp
@@ -8,13 +7,10 @@ import subprocess as sp
 import pytest
 
 import xonsh
-from xonsh.platform import ON_WINDOWS
 from xonsh.lib.os import indir
 
 from tools import (
     skip_if_on_windows,
-    skip_if_on_darwin,
-    skip_if_on_travis,
     ON_WINDOWS,
     ON_DARWIN,
     ON_TRAVIS,
@@ -312,6 +308,34 @@ echo @$(which ls)
         0,
     ),
     #
+    # test @$() without leading/trailig WS
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo foo_@$(echo spam)_bar
+""",
+        "foo_spam_bar\n",
+        0,
+    ),
+    #
+    # test @$() outer product
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+echo foo_@$(echo spam sausage)_bar
+""",
+        "foo_spam_bar foo_sausage_bar\n",
+        0,
+    ),
+    #
     # test redirection
     #
     (
@@ -347,6 +371,50 @@ del _f
         "",
         0,
     ),
+    #
+    # test single check_output
+    #
+    (
+        """
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+
+from xonsh.lib.subprocess import check_output
+
+print(check_output(["echo", "hello"]).decode("utf8"))
+""",
+        "hello\n\n",
+        0,
+    ),
+    #
+    # test contextvars
+    #
+    (
+        """
+import sys
+
+if sys.version_info[:2] >= (3, 7):
+    with open("sourced-file.xsh", "w") as f:
+        f.write('''
+from contextvars import ContextVar
+
+var = ContextVar('var', default='spam')
+var.set('foo')
+        ''')
+
+    source sourced-file.xsh
+
+    print("Var " + var.get())
+
+    import os
+    os.remove('sourced-file.xsh')
+else:
+    print("Var foo")
+""",
+        "Var foo\n",
+        0,
+    ),
 ]
 
 
@@ -485,6 +553,7 @@ def test_redirect_out_to_file(cmd, exp, tmpdir):
 @skip_if_no_xonsh
 @skip_if_no_sleep
 @skip_if_on_windows
+@pytest.mark.xfail(strict=False) # TODO: fixme (super flaky on OSX)
 def test_xonsh_no_close_fds():
     # see issue https://github.com/xonsh/xonsh/issues/2984
     makefile = (
@@ -503,3 +572,43 @@ def test_xonsh_no_close_fds():
             f.write(makefile)
         out = sp.check_output(["make", "-sj2", "SHELL=xonsh"], universal_newlines=True)
         assert "warning" not in out
+
+
+@pytest.mark.parametrize(
+    "cmd, fmt, exp", [("ls | wc", lambda x: x > "", True),],  # noqa E231 (black removes space)  
+)
+def test_pipe_between_subprocs(cmd, fmt, exp):
+    "verify pipe between subprocesses doesn't throw an exception"
+    check_run_xonsh(cmd, fmt, exp)
+
+
+@skip_if_on_windows
+def test_negative_exit_codes_fail():
+    # see issue 3309
+    script = 'python -c "import os; os.abort()" && echo OK\n'
+    out, err, rtn = run_xonsh(script)
+    assert "OK" != out
+    assert "OK" != err
+
+
+@pytest.mark.parametrize(
+    "cmd, exp",
+    [
+        ("echo '&'", "&\n"),
+        ("echo foo'&'", "foo'&'\n"),
+        ("echo foo '&'", "foo &\n"),
+        ("echo foo '&' bar", "foo & bar\n"),
+    ],
+)
+def test_ampersand_argument(cmd, exp):
+    script = """
+#!/usr/bin/env xonsh
+def _echo(args):
+    print(' '.join(args))
+aliases['echo'] = _echo
+{}
+""".format(
+        cmd
+    )
+    out, _, _ = run_xonsh(script)
+    assert out == exp
