@@ -7,11 +7,11 @@ import itertools
 
 import pytest
 
-from xonsh.ast import AST, With, Pass, pdump
+from xonsh.ast import AST, With, Pass, Str, Call
 from xonsh.parser import Parser
 from xonsh.parsers.base import eval_fstr_fields
 
-from tools import VER_FULL, skip_if_py34, skip_if_lt_py36, nodes_equal
+from tools import VER_FULL, skip_if_lt_py36, nodes_equal
 
 # a lot of col_offset data changed from Py v3.5.0 -> v3.5.1
 INC_ATTRS = (3, 5, 1) <= VER_FULL
@@ -186,7 +186,6 @@ def test_binop_times():
     check_ast("42 * 65")
 
 
-@skip_if_py34
 def test_binop_matmult():
     check_ast("x @ y", False)
 
@@ -682,57 +681,46 @@ def test_dict_three():
     check_ast("{42: 65, 6: 28, 1: 2}")
 
 
-@skip_if_py34
 def test_dict_from_dict_two_xy():
     check_ast('{"x": 1, **{"y": 2}}')
 
 
-@skip_if_py34
 def test_dict_from_dict_two_x_first():
     check_ast('{"x": 1, **{"x": 2}}')
 
 
-@skip_if_py34
 def test_dict_from_dict_two_x_second():
     check_ast('{**{"x": 2}, "x": 1}')
 
 
-@skip_if_py34
 def test_unpack_range_tuple():
     check_stmts("*range(4),")
 
 
-@skip_if_py34
 def test_unpack_range_tuple_4():
     check_stmts("*range(4), 4")
 
 
-@skip_if_py34
 def test_unpack_range_tuple_parens():
     check_ast("(*range(4),)")
 
 
-@skip_if_py34
 def test_unpack_range_tuple_parens_4():
     check_ast("(*range(4), 4)")
 
 
-@skip_if_py34
 def test_unpack_range_list():
     check_ast("[*range(4)]")
 
 
-@skip_if_py34
 def test_unpack_range_list_4():
     check_ast("[*range(4), 4]")
 
 
-@skip_if_py34
 def test_unpack_range_set():
     check_ast("{*range(4)}")
 
 
-@skip_if_py34
 def test_unpack_range_set_4():
     check_ast("{*range(4), 4}")
 
@@ -1097,17 +1085,14 @@ def test_call_dict_kwargs():
     check_ast('dict(**{"base": 8})')
 
 
-@skip_if_py34
 def test_call_list_many_star_args():
     check_ast("min(*[1, 2], 3, *[4, 5])")
 
 
-@skip_if_py34
 def test_call_list_many_starstar_args():
     check_ast('dict(**{"a": 2}, v=3, **{"c": 5})')
 
 
-@skip_if_py34
 def test_call_list_many_star_and_starstar_args():
     check_ast('x(*[("a", 2)], *[("v", 3)], **{"c": 5})', False)
 
@@ -1253,7 +1238,6 @@ def test_times_eq():
     check_stmts("x = 42; x *= 2")
 
 
-@skip_if_py34
 def test_matmult_eq():
     check_stmts("x @= y", False)
 
@@ -1649,7 +1633,6 @@ def test_for_else():
     check_stmts("for x in range(6):\n  pass\nelse:  pass")
 
 
-@skip_if_py34
 def test_async_for():
     check_stmts("async def f():\n    async for x in y:\n        pass\n", False)
 
@@ -1678,7 +1661,6 @@ def test_with_in_func():
     check_stmts("def f():\n    with x:\n        pass\n")
 
 
-@skip_if_py34
 def test_async_with():
     check_stmts("async def f():\n    async with x as y:\n        pass\n", False)
 
@@ -2014,17 +1996,14 @@ def test_function_blank_line():
     check_stmts(code, False)
 
 
-@skip_if_py34
 def test_async_func():
     check_stmts("async def f():\n  pass\n")
 
 
-@skip_if_py34
 def test_async_decorator():
     check_stmts("@g\nasync def f():\n  pass", False)
 
 
-@skip_if_py34
 def test_async_await():
     check_stmts("async def f():\n    await fut\n", False)
 
@@ -2271,6 +2250,13 @@ def test_rhs_nested_injection():
     check_xonsh_ast({}, "$[ls @$(dirname @$(which python))]", False)
 
 
+def test_merged_injection():
+    tree = check_xonsh_ast({}, "![a@$(echo 1 2)b]", False, return_obs=True)
+    assert isinstance(tree, AST)
+    func = tree.body.args[0].right.func
+    assert func.attr == "list_of_list_of_strs_outer_product"
+
+
 def test_backtick_octothorpe():
     check_xonsh_ast({}, "print(`#.*`)", False)
 
@@ -2720,7 +2706,6 @@ def test_arg_single_subprocbang(opener, closer, body):
     "body", ["echo -n!x", "echo -n!x", "echo -n !x", "echo -n ! x"]
 )
 def test_arg_single_subprocbang_nested(opener, closer, ipener, iloser, body):
-    code = opener + "echo " + ipener + body + iloser + closer
     tree = check_xonsh_ast({}, opener + body + closer, False, return_obs=True)
     assert isinstance(tree, AST)
     cmd = tree.body.args[0].elts
@@ -2889,6 +2874,20 @@ def test_withbang_as_many_suite(body):
         assert s == body
 
 
+def test_subproc_raw_str_literal():
+    tree = check_xonsh_ast({}, "!(echo '$foo')", run=False, return_obs=True)
+    assert isinstance(tree, AST)
+    subproc = tree.body
+    assert isinstance(subproc.args[0].elts[1], Call)
+    assert subproc.args[0].elts[1].func.attr == "expand_path"
+
+    tree = check_xonsh_ast({}, "!(echo r'$foo')", run=False, return_obs=True)
+    assert isinstance(tree, AST)
+    subproc = tree.body
+    assert isinstance(subproc.args[0].elts[1], Str)
+    assert subproc.args[0].elts[1].s == "$foo"
+
+
 # test invalid expressions
 
 
